#'ADAMONT QQCorr computes quantile-quantile correction of seasonal or decadal forecast data
#'
#'@author Paola Marson, \email{paola.marson@meteo.fr} for PROSNOW version
#'@author Lauriane Batté, \email{lauriane.batte@meteo.fr} for CSTools adaptation
#'
#'@param exp dimensions c(dataset,member,sdate,ftime,lat,lon)
#'@param wt dimensions c(dataset,member,sdate,ftime)
#'@param qexp dimensions c(Nwt, Nq+2, nlat, nlon)
#'@param qobs dimensions c(Nwt, Nq+2, nlat_o, nlon_o)
#'@param NN list (output from Adamont_NearestNeighbors) maps (nlat, nlon) onto (nlat_o, nlon_o)
#'
#'@import qmap
#'@import multiApply
#'@import abind

.QQSlope <- function(qexp, qobs) {
   Nwt <- dim(qexp)[1]
   Nq <- dim(qexp)[2] - 2
   slope1 <- (qobs[,2]-qobs[,1])/(qexp[,2]-qexp[,1])
   slope2 <- matrix(1,nrow=Nwt,ncol=Nq+2)
   return(list(s1=slope1,s2=slope2))
}

AdamontQQCorr <- function(exp, wt, obs, wt_obs, NN, wetonly=FALSE, thres=1.,corrdims = c('member', 'sdate', 'ftime'), londim='lon', latdim='lat') {

   obsdims <- names(dim(obs$data))
   poslat <- which(obsdims == latdim)
   poslon <- which(obsdims == londim)
   nlat_o <- dim(obs$data)[poslat]
   nlon_o <- dim(obs$data)[poslon]
   ilat_o <- array(c(1:nlat_o))
   dim(ilat_o) <- c('lat') 
   ilon_o <- array(c(1:nlon_o))
   dim(ilon_o) <- c('lon') 
   # First step if obs data is higher resolution than exp data is to use nearest neighbor
   # to compute downscaling of exp data	
   exp_corr <- Apply(list(exp$data,ilat_o,ilon_o),target_dims=list(c(londim,latdim),'lat','lon'),.getNN,NN=NN)$output1

   # Reorder exp_corr dimensions to match exp dimensions


   # Both exp and obs data are now on the same grid
   # Use CST_QuantileMapping function for quantile mapping, depending on weather type
   #test_qmap <- CST_QuantileMapping(exp=mod,obs=obs,sample_dims=c('dataset','member','sdate','ftime'),method='RQUANT')
   # Reshape obs and exp data
   # Apply QuantileMapping with exp_cor output for each weather type
   

   # Reshaping obs and exp_corr data
    

   return(exp_corr)
}

.getNN <- function(exp,ilat,ilon,NN){
   return(exp[NN$imin_lat[ilat,ilon],NN$imin_lon[ilat,ilon]])
}

